@@ -1,25 +1,40 @@
 ;;; LIBSVM interface
 ;;; <http://www.csie.ntu.edu.tw/~cjlin/libsvm/>
 ;;;
-;;; Copyright (C) 2006, 2007 Sam Steingold <sds@gnu.org>
+;;; Copyright (C) 2006-2010 Sam Steingold <sds@gnu.org>
 ;;; This is Free Software, covered by the GNU GPL (v2)
 ;;; See http://www.gnu.org/copyleft/gpl.html
 
 (defpackage "LIBSVM"
   (:modern t) (:use "CL" "FFI")
   (:shadowing-import-from "EXPORTING"
-    #:def-c-enum #:def-c-struct #:def-call-out #:def-c-type #:defun))
+    #:def-c-enum #:def-c-struct #:def-call-out #:def-c-type #:def-c-var
+    #:defun #:defvar))
 (in-package "LIBSVM")
 (setf (documentation (find-package "LIBSVM") 'sys::impnotes) "libsvm")
 
 (default-foreign-language :stdc)
-(default-foreign-library
- (namestring (merge-pathnames "svm.so" *load-pathname*)))
+
+(c-lines "#include \"config.h\"~%#include <svm.h>~%")
+
+(defvar *libsvm-output* *standard-output* "The stream for svm.so messages.")
+(cl:defun write-string-to-libsvm-output (s)
+  (write-string s *libsvm-output*) (force-output *libsvm-output*))
+(def-call-in write-string-to-libsvm-output (:name "libsvm_print_string")
+  (:arguments (s c-string)) (:return-type nil))
+(c-lines :init-always "
+ #if defined(HAVE_SVM_SET_PRINT_STRING_FUNCTION)
+  svm_set_print_string_function((void (*) (const char*))&libsvm_print_string);
+ #else
+  svm_print_string = (void (*) (const char*))&libsvm_print_string;
+ #endif~%")
 
 ;;;
 ;;; types and constants
 ;;;
 
+(def-c-var *libsvm-version* (:name "libsvm_version") (:type int))
+
 (def-c-type node (c-struct list (index int) (value double-float)))
 
 (def-c-type problem (c-struct list
