@@ -1,5 +1,5 @@
 ;;; handle the posix functions
-;;; Sam Steingold 1999-2009
+;;; Sam Steingold 1999-2010
 
 (defpackage #:posix
   (:use #:common-lisp #:ext)
@@ -9,7 +9,8 @@
   (:export
    #:resolve-host-ipaddr #:bogomips #:loadavg #:mkstemp #:mkdtemp #+unix #:wait
    #:stream-lock #:with-stream-lock #:duplicate-handle #:copy-file
-   #:file-owner #:physical-memory #:stream-options #+unix #:string-time
+   #:file-owner #:physical-memory #:stream-options #:string-time
+   #:version-compare #:version< #:version<= #:version> #:version>=
    #+(or :win32 :cygwin) #:file-properties #+unix #:make-xterm-io-stream
    #:priority #:openlog #:setlogmask #:syslog #:closelog #:process-id #:getppid
    #:getsid #:setsid #:getpgrp #:setpgrp #:setreuid #:setregid #:kill #:sync
@@ -134,7 +135,6 @@
   (fs-type nil :type (or null string) :read-only t))
 ;;; ============================================================
 #+unix (export '(sysconf confstr pathconf))
-#+unix
 (defstruct (uname (:constructor make-uname (sysname nodename release
                                             version machine)))
   "see uname(2) for details"
@@ -203,7 +203,8 @@
                     (file-stat-file file-stat)))) ; name-short
 ;;; ============================================================
 #+(or win32 cygwin) (progn
-(export '(make-shortcut))
+(export '(make-shortcut get-user-sid clipboard))
+(defsetf clipboard %set-clipboard)
 
 (defstruct (shortcut-info
              (:constructor make-shortcut-info
