@@ -2,7 +2,7 @@
 ;;;; Generic Functions
 ;;;; Part 2: Generic function dispatch and execution
 ;;;; Bruno Haible 21.8.1993 - 2004
-;;;; Sam Steingold 1998-2004, 2007-2008
+;;;; Sam Steingold 1998-2004, 2007-2008, 2010
 ;;;; German comments translated into English: Stefan Kain 2002-04-08
 
 (in-package "CLOS")
@@ -547,8 +547,8 @@
   ;; Check the lambda-list.
   (let* ((signature
            (generic-function-lambda-list-to-signature lambdalist
-             #'(lambda (detail errorstring &rest arguments)
-                 (funcall errfunc detail
+             #'(lambda (lalist detail errorstring &rest arguments)
+                 (funcall errfunc lalist detail
                           (TEXT "Invalid generic function lambda-list: ~A")
                           (apply #'format nil errorstring arguments)))))
          (reqnum (sig-req-num signature))
