@@ -1,6 +1,6 @@
 ;;;; Common Lisp Object System for CLISP: Generic Functions
 ;;;; Bruno Haible 21.8.1993 - 2004
-;;;; Sam Steingold 1998 - 2004, 2007
+;;;; Sam Steingold 1998 - 2004, 2007 - 2010
 ;;;; German comments translated into English: Stefan Kain 2002-04-08
 
 (in-package "CLOS")
@@ -103,11 +103,13 @@
   (unless (defined-class-p generic-function-class)
     (if (symbolp generic-function-class)
       (setq generic-function-class (find-class generic-function-class))
-      (error (TEXT "~S for generic-function ~S: generic-function-class ~S is neither a class or a symbol")
-             'ensure-generic-function-using-class funname generic-function-class)))
+      (error (TEXT "~S for generic-function ~S: ~S ~S is neither a class or a symbol")
+             'ensure-generic-function-using-class funname
+             ':generic-function-class generic-function-class)))
   (unless (subclassp generic-function-class <generic-function>)
-    (error (TEXT "~S for generic-function ~S: generic-function-class ~S is not a subclass of GENERIC-FUNCTION")
-           'ensure-generic-function-using-class funname generic-function-class))
+    (error (TEXT "~S for generic-function ~S: ~S ~S is not a subclass of ~S")
+           'ensure-generic-function-using-class funname
+           ':generic-function-class generic-function-class 'GENERIC-FUNCTION))
   ;; Preparation of initialization arguments.
   (setq all-keys (copy-list all-keys))
   (remf all-keys ':generic-function-class)
@@ -115,8 +117,9 @@
     (unless (defined-class-p method-class)
       (if (symbolp method-class)
         (setq method-class (find-class method-class))
-        (error (TEXT "~S for generic-function ~S: method-class ~S is neither a class or a symbol")
-               'ensure-generic-function-using-class funname method-class)))
+        (error (TEXT "~S for generic-function ~S: ~S ~S is neither a class or a symbol")
+               'ensure-generic-function-using-class funname
+               ':method-class method-class)))
     (setf (getf all-keys ':method-class) method-class))
   (if gf
     ;; Redefinition of a generic function.
@@ -213,8 +216,8 @@
   (multiple-value-bind (fast-function-factory-lambda method-initargs-forms signature)
       (analyze-method-description 'defmethod whole-form funname method-description)
     `(LET ()
-       (COMPILER::EVAL-WHEN-COMPILE
-         (COMPILER::C-DEFUN ',funname ,signature nil 'DEFMETHOD))
+       (SYS::EVAL-WHEN-COMPILE
+         (SYS::C-DEFUN ',funname ,signature nil 'DEFMETHOD))
        (WHEN (GET ',(SYS::GET-FUNNAME-SYMBOL funname) 'SYS::TRACED-DEFINITION)
          (SYS::UNTRACE1 ',funname))
        (DO-DEFMETHOD ',funname (FUNCTION ,fast-function-factory-lambda)
@@ -269,8 +272,8 @@
   (multiple-value-bind (fast-function-factory-lambda method-initargs-forms signature)
       (analyze-method-description 'defmethod whole-form funname method-description)
     (declare (ignore fast-function-factory-lambda method-initargs-forms))
-    `(COMPILER::EVAL-WHEN-COMPILE
-       (COMPILER::C-DEFUN ',funname ,signature nil 'DEFMETHOD))))
+    `(SYS::EVAL-WHEN-COMPILE
+       (SYS::C-DEFUN ',funname ,signature nil 'DEFMETHOD))))
 
 
 ;; ====================== DEFGENERIC and similar Macros ======================
@@ -332,8 +335,7 @@
          |#
          (check-gf-declspecs (rest option) 'declare
            #'(lambda (errorstring &rest arguments)
-               (error (TEXT "~S ~S: ~A")
-                      caller funname
+               (error "~S ~S: ~A" caller funname
                       (apply #'format nil errorstring arguments))))
          (setq declares
                (if declares
@@ -426,7 +428,7 @@
          (error-of-type 'ext:source-program-error
            :form whole-form
            :detail option
-           (TEXT "~S ~S: invalid ~S option: ~S")
+           (TEXT "~S ~S: invalid ~S option ~S")
            caller funname 'defgeneric option))
         (t
          (let ((optionkey (first option)))
@@ -450,8 +452,8 @@
               (error-of-type 'ext:source-program-error
                 :form whole-form
                 :detail detail
-                (TEXT "~S ~S: ~A")
-                caller funname (apply #'format nil errorstring arguments))))
+                "~S ~S: ~A" caller funname
+                (apply #'format nil errorstring arguments))))
       (declare (ignore argorder))
       (let ((method-combinations-defaulted
               ;; Default :method-combination is STANDARD:
@@ -507,10 +509,9 @@
 (defun analyze-defgeneric-lambdalist (caller whole-form funname lambdalist)
   (multiple-value-bind (reqvars optvars rest keyp keywords keyvars allowp)
       (sys::analyze-generic-function-lambdalist lambdalist
-        #'(lambda (detail errorstring &rest arguments)
-            (error-of-type 'ext:source-program-error
-              :form whole-form
-              :detail detail
+        #'(lambda (lalist detail errorstring &rest arguments)
+            (declare (ignore lalist)) ; use WHOLE-FORM instead
+            (sys::lambda-list-error whole-form detail
               (TEXT "~S ~S: invalid generic function lambda-list: ~A")
               caller funname (apply #'format nil errorstring arguments))))
     (declare (ignore keyvars))
@@ -531,8 +532,8 @@
            (method-combination-form `(,method-combination-lambda ,generic-function-class-var)))
       `(LET ()
          (DECLARE (SYS::IN-DEFUN ,funname))
-         (COMPILER::EVAL-WHEN-COMPILE
-           (COMPILER::C-DEFUN ',funname ',signature nil 'DEFGENERIC))
+         (SYS::EVAL-WHEN-COMPILE
+           (SYS::C-DEFUN ',funname ',signature nil 'DEFGENERIC))
          (WHEN (GET ',funname-symbol 'SYS::TRACED-DEFINITION)
            (SYS::UNTRACE1 ',funname))
          ;; NB: no (SYSTEM::REMOVE-OLD-DEFINITIONS ',funname)
