@@ -1,6 +1,6 @@
 ;;;; Common Lisp Object System for CLISP: Methods
 ;;;; Bruno Haible 21.8.1993 - 2004
-;;;; Sam Steingold 1998 - 2004, 2008
+;;;; Sam Steingold 1998 - 2004, 2008, 2010
 ;;;; German comments translated into English: Stefan Kain 2002-04-08
 
 (in-package "CLOS")
@@ -31,7 +31,7 @@
               (push (first item) req-vars)
               (push (first item) ignorable-req-vars) ; CLtL2 p. 840 top
               (push (second item) spec-list))
-            (funcall errfunc item
+            (funcall errfunc specialized-lambda-list item
               (TEXT "Invalid specialized parameter in method lambda list ~S: ~S")
               specialized-lambda-list item)))))
     (let ((lambda-list (nreconc req-vars remaining-lambda-list)))
@@ -40,10 +40,9 @@
 
 ;; helper
 (defmacro program-error-reporter (caller)
-  `#'(lambda (detail errorstring &rest arguments)
-       (declare (ignore detail))
-       (error-of-type 'program-error "~S: ~A" ,caller
-         (apply #'format nil errorstring arguments))))
+  `#'(lambda (form detail errorstring &rest arguments)
+       (apply #'sys::lambda-list-error form detail
+              "~S: ~A" ,caller (apply #'format nil errorstring arguments))))
 
 ;; MOP p. 52
 (defun extract-lambda-list (specialized-lambda-list)
@@ -90,10 +89,9 @@
           (body (cdr description)))
       (multiple-value-bind (lambda-list spec-list ignorable-req-vars)
           (decompose-specialized-lambda-list specialized-lambda-list
-            #'(lambda (detail errorstring &rest arguments)
-                (error-of-type 'ext:source-program-error
-                  :form whole-form
-                  :detail detail
+            #'(lambda (form detail errorstring &rest arguments)
+                (declare (ignore form)) ; FORM is lambda-list, use WHOLE-FORM
+                (sys::lambda-list-error whole-form detail
                   "~S ~S: ~A" caller funname
                   (apply #'format nil errorstring arguments))))
         (let ((req-specializer-forms
@@ -118,10 +116,9 @@
                                 keyp keywords keyvars keyinits keysvars
                                 allowp auxvars auxinits)
               (analyze-lambdalist lambda-list
-                #'(lambda (detail errorstring &rest arguments)
-                    (error-of-type 'ext:source-program-error
-                      :form whole-form
-                      :detail detail
+                #'(lambda (lalist detail errorstring &rest arguments)
+                    (declare (ignore lalist)) ; use WHOLE-FORM instead
+                    (sys::lambda-list-error whole-form detail
                       "~S ~S: ~A" caller funname
                       (apply #'format nil errorstring arguments))))
             (declare (ignore optinits optsvars keyvars keyinits keysvars
