@@ -36,15 +36,15 @@ Syntax eines Inputfile:
 
 int main (int argc, char* argv[]) {
   char infilenamebuffer[1000];
-  char outfilenamebuffer[1000];
+  char outfilenamebuffer[1000] = "stdout";
   FILE * infile;
   FILE * outfile;
   if (argc==3) { /* Aufruf der Form 'comment source destination' */
     { char* p1 = argv[1]; char* p2 = infilenamebuffer;
-      while ((*p2++ = *p1++) != '\0');
+      while ((*p2++ = *p1++) != '\0') {}
     }
     { char* p1 = argv[2]; char* p2 = outfilenamebuffer;
-      while ((*p2++ = *p1++) != '\0');
+      while ((*p2++ = *p1++) != '\0') {}
     }
   } else {
     char filenamebuffer[1000];
@@ -116,21 +116,21 @@ int main (int argc, char* argv[]) {
          if (!(c=='\n')) { fputc('\\',outfile); goto L2a; }
          fput_endcomment(outfile); fputc('\\',outfile); fputc(c,outfile);
          goto L1;
-    L4a: /* possibly start of '/*' comment */
+    L4a: /* possibly start of '／＊' comment */
          /* EOF impossible */
          fputc(c,outfile);
-         if (!(c=='/')) {  goto L1; } 
+         if (!(c=='/')) {  goto L1; }
          c = fgetc(infile) ;
          if (c==EOF){ goto L3; }
          fputc(c,outfile);
          if (!(c=='*')) goto L1;
-    L5:  /* Inside '/*' comment */
+    L5:  /* Inside '／＊' comment */
          c = fgetc(infile) ;
     L5a: if (c==EOF){ goto L3; } /* do not fix programmer errors */
          fputc(c,outfile);
          if (c=='*') goto L6;
          goto L5;
-    L6:  /* after asterisk in '/*' comment */
+    L6:  /* after asterisk in '／＊' comment */
          c = fgetc(infile) ;
     L6a: if (c==EOF){ goto L3; }
          fputc(c,outfile);
@@ -139,13 +139,10 @@ int main (int argc, char* argv[]) {
     L3:  ; /* am File-Ende */
   }
   /* error checking should work after file closing, but it does not */
-  if (ferror(infile))
+  if (ferror(infile) || fclose(infile))
     { perror(infilenamebuffer); exit(__LINE__); }
-  if (ferror(outfile))
+  if (ferror(outfile) || fclose(outfile))
     { perror(outfilenamebuffer); exit(__LINE__); }
-  /* close files: */
-  fclose(infile);
-  fclose(outfile);
   exit(0); /* OK */
 }
 
