@@ -1,8 +1,8 @@
 ;;; MODPREP - CLISP module preprocessor
 ;;;
 ;;; Copyright (C) 1998 Bruno Haible (20.9.1998, 10.-11.10.1998) [C]
-;;; Copyright (C) 2003-2008 by Sam Steingold [lisp]
-;;; This is Free Software, covered by the GNU GPL (v2)
+;;; Copyright (C) 2003-2010 by Sam Steingold [lisp]
+;;; This is Free Software, covered by the GNU GPL (v2+)
 ;;; See http://www.gnu.org/copyleft/gpl.html
 #| This preprocessor generates all necessary tables for a CLISP module.
 The input file is normal C code, modified like this:
@@ -18,7 +18,9 @@ The input file is normal C code, modified like this:
   The backquoted strings are read in at module load time.
 - The module SUBRs can be referred to using double backquote syntax:
     pushSTACK(``FOO:BAR``);
-  means the same as L(bar) in the CLISP sources
+  means the same as L(bar) in the CLISP sources.
+- CPP definienda ("#define FOO baz zot") can be accessed as strings using
+    `STRINGIFY(FOO)` which evaluates to the Lisp string "baz zot".
 - The definition of Lisp functions is done using the macro
     DEFUN(function_name, lambda_list)
   for example
@@ -335,7 +337,10 @@ FOO(bar,baz,zot) ==> FOO; (bar baz zot); end-position"
   (find tag *objdefs* :test #'string= :key #'objdef-tag))
 (defun write-string-c-style (string out)
   (loop :for cc :across string :and ii :upfrom 0
-    :do (cond ((alphanumericp cc) (write-char (char-downcase cc) out))
+    :do (cond ((alphanumericp cc)
+               (write-char (if (case-sensitive-package-p)
+                               cc (char-downcase cc))
+                           out))
               ((char= cc #\:) (write-char (if (zerop ii) #\K #\_) out))
               ((or (char= cc #\_) (char= cc #\-)) (write-char #\_ out))
               (t (format out "_~2,'0x" (char-code cc))))))
@@ -373,6 +378,9 @@ See bug #[ 1491252 ]: i18n does not build on cf:alpha")
 (defun case-sensitive-package-p ()
   (let ((properties (gethash *module-package* *package-properties*)))
     (or (getf properties :modern) (getf properties :case-sensitive))))
+(defun case-inverted-package-p ()
+  (let ((properties (gethash *module-package* *package-properties*)))
+    (or (getf properties :modern) (getf properties :case-inverted))))
 
 (defun string-upcase-verbose (string)
   (when (and (some #'lower-case-p string)    ; upcasing will modify
@@ -597,8 +605,9 @@ The last feature is disabled because &S() does not work in non-debug builds."
       (setq pack (string-upcase pack)))
     (unless (every (lambda (c) (or (not (alpha-char-p c)) (upper-case-p c)))
                    name)
-      (warn "~S:~D: fixed function name case ~S" *input-file* *lineno* name)
-      (setq name (string-upcase name)))
+      (unless (case-sensitive-package-p)
+        (warn "~S:~D: fixed function name case ~S" *input-file* *lineno* name)
+        (setq name (string-upcase name))))
     (pushnew pack *module-all-packages* :test #'string=)
     (vector-push-extend fd *fundefs*)
     fd))
@@ -1056,7 +1065,10 @@ commas and parentheses."
       :do (with-conditional (out (objdef-cond-stack od))
             (let ((init (objdef-init od)))
               (etypecase init
-                (string (format out "  { ~S }," init))
+                (string (if (let ((pos (position #\( init)))
+                              (and pos (string= init "STRINGIFY" :end1 pos)))
+                            (format out "  { \"\\\"~S\\\"\" }," init)
+                            (format out "  { ~S }," init)))
                 (cons (format out "  {~%")
                       (dolist (el init)
                         (etypecase el
@@ -1173,7 +1185,11 @@ commas and parentheses."
     (formatln out "} ~A = {" subr-tab-initdata)
     (loop :for fd :across *fundefs*
       :do (with-conditional (out (fundef-cond-stack fd))
-            (format out "  { ~S, ~S }," (fundef-pack fd) (fundef-name fd))))
+            (format out "  { ~S, ~S }," (fundef-pack fd)
+                    (if (case-inverted-package-p)
+                        ;; will be inverted again lisp-side
+                        (string-invertcase (fundef-name fd))
+                        (fundef-name fd)))))
     (formatln out "  0")
     (formatln out "};") (newline out)
     ;; Emit the decl first, to avoid "gcc -missing-declarations" warnings.
@@ -1218,11 +1234,39 @@ commas and parentheses."
       (print-tables-1 out)))
   (print-tables-2 out))
 
-(defun mod-file (input)
+(defun mod-file-1 (input &optional (defaults input))
   (make-pathname :name (ext:string-concat (pathname-name input) ".m")
-                 :defaults input))
+                 :defaults defaults))
 
-(defun modprep (*input-file* &optional (output (mod-file *input-file*)))
+(defun mod-file (input &optional (output t)
+                 &aux (sys::*internal-compiled-file-type*
+                       (pathname-type input)))
+  (let ((output1 (compile-file-pathname input :output-file output)))
+    (unless (and (or (pathnamep output) (stringp output))
+                 (pathname-name output))
+      (setq output1 (mod-file-1 output1)))
+    (when (string= (namestring (absolute-pathname output1))
+                   (namestring (absolute-pathname input)))
+      (error "~S(~S, ~S): output ~S the same as input"
+             'mod-file input output output1))
+    output1))
+#|
+ (dolist (a '(("/foo/bar/zot.m.c" "/foo/bar/zot.c")
+              ("/quux/froq.a" "/foo/bar/zot.c" "/quux/froq.a")
+              ("zot.m.c" "/foo/bar/zot.c" "./")
+              ("here/zot.m.c" "/foo/bar/zot.c" "./here/")
+              ("quux.c" "/foo/bar/zot.c" "./quux.c")
+              ("foo/bar/zot.m.c" "foo/bar/zot.c")
+              ("/quux/froq.a" "foo/bar/zot.c" "/quux/froq.a")
+              ("zot.m.c" "foo/bar/zot.c" "./")
+              ("here/zot.m.c" "foo/bar/zot.c" "./here/")
+              ("quux.c" "foo/bar/zot.c" "./quux.c")))
+   (let* ((expected (pop a)) (actual (apply #'mod-file a)))
+     (format t "~&~S --> ~S [~S ~A]~%"
+             a actual expected (equal expected (namestring actual)))))
+|#
+
+(defun modprep (*input-file* &optional (output (mod-file-1 *input-file*)))
   (format t "~&;; ~S: ~S --> ~S~%" 'modprep *input-file* output)
   (with-open-file (in *input-file* :direction :input
                       #+UNICODE :external-format #+UNICODE charset:utf-8)
@@ -1243,6 +1287,6 @@ commas and parentheses."
     (format t "~&~S: wrote ~A (~:D byte~:P)~&"
             'modprep output (file-length out))))
 
-(modprep (first *args*) (or (second *args*) (mod-file (first *args*))))
+(modprep (first *args*) (apply #'mod-file *args*))
 
 ;;; file modprep.lisp ends here
